import psycopg2
import pandas as pds
import logging

logger = logging.getLogger(__name__)


class AircloakConnection():
    def __init__(self, *, host: str, port: int, user: str, password: str, dataset: str):
        logger.debug(f'Aircloak: Connecting to {dataset} @ {host}:{port}.')
        self.host = host
        self.port = port
        self.dataset = dataset

        try:
            self.conn = psycopg2.connect(
                user=user,
                host=host,
                port=port,
                dbname=dataset,
                password=password)
        except psycopg2.OperationalError as err:
            logger.error(f'internal psycopg2 error: {err}')
            raise

    def close(self):
        logger.debug('Closing Aircloak connection.')
        self.conn.close()

    def is_connected(self):
        return self.conn.closed == 0

    def query(self, statement: str) -> pds.DataFrame:
        logger.debug(f'Sending query {statement}...')
        df = pds.read_sql_query(statement, self.conn)
        logger.debug(f'Got result: {df[:5]}')

        return df

    def get_tables(self) -> pds.DataFrame:
        return self.query("SHOW TABLES")

    def get_columns_for_table(self, table) -> pds.DataFrame:
        return self.query(f'SHOW COLUMNS FROM {table}')

    def count_distinct_values(self, table, column, order_by='count') -> pds.DataFrame:
        return self.query(f'''
            SELECT {column}, count(*) as count
            FROM {table} 
            GROUP BY {column}
            ORDER BY {order_by} DESC''')



__version__ = '0.1.1'

from aircloak_tools import aircloak_client
from contextlib import contextmanager


@contextmanager
def connect(host, port, user, password, dataset):
    try:
        ac = aircloak_client.AircloakConnection(
            host=host, port=port, user=user, password=password, dataset=dataset)
        yield ac
    finally:
        ac.close()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aircloak_tools']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.0.4,<2.0.0', 'psycopg2>=2.8.5,<3.0.0']

setup_kwargs = {
    'name': 'aircloak-tools',
    'version': '0.1.1',
    'description': 'Tools for querying an Aircloak service via its postgres api.',
    'long_description': '# Python Aircloak Tools\n\nA small package for querying an Aircloak service via the postgres api. \n\nThe main aim is to provide an Aircloak-friendly wrapper around `psycopg2`, and in particular to\nprovide clear error messages when something doesn\'t go as planned. \n\nQuery results are return as `pandas` dataframes. \n\n\n## Example\n\nThe following code shows how to initiate a connection and execute a query.\n\nAs a pre-requisite you should have a username and password for the postgres interface of an\nAircloak installation (ask your admin for these). Assign these values to `AIRCLOAK_PG_USER`\nand `AIRCLOAK_PG_PASSWORD` environment variables. \n\n> Note the call to ``ac.connect()`` can be used as a context manager: Using the ``with`` statement, the connection \n> is automatically closed cleanly when it it goes out of scope.\n\n```python\nimport aircloak_tools as ac\n\nAIRCLOAK_PG_HOST = "covid-db.aircloak.com"\nAIRCLOAK_PG_PORT = 9432\n\nAIRCLOAK_PG_USER = environ.get("AIRCLOAK_PG_USER")\nAIRCLOAK_PG_PASSWORD = environ.get("AIRCLOAK_PG_PASSWORD")\n\nTEST_DATASET = "cov_clear"\n\nwith ac.connect(host=AIRCLOAK_PG_HOST, port=AIRCLOAK_PG_PORT,\n                user=AIRCLOAK_PG_USER, password=AIRCLOAK_PG_PASSWORD, dataset=TEST_DATASET) as conn:\n\n    assert(conn.is_connected())\n\n    tables = conn.get_tables()\n\n    print(tables)\n\n    feeling_now_counts = conn.query(\'\'\'\n    select feeling_now, count(*), count_noise(*)\n    from survey\n    group by 1\n    order by 1 desc\n    \'\'\')\n```\n',
    'author': 'dlennon',
    'author_email': '3168260+dandanlen@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
